# -*- coding: utf-8 -*-

# Copyright 2016-2017 Mads Michelsen (mail@brokkr.net)
# This file is part of Derailleur.
# Derailleur is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License,
# or (at your option) any later version.

"""Parses the configuration and validates it against an external schema"""

import sys
import fileinput
from os import path, makedirs
from configobj import ConfigObj
from validate import Validator, ValidateError
from derailleur import __file__ as module_file


def nestring_check(value):
    '''Requires string to not be empty or contain whitespace'''
    if not isinstance(value, (str, int)):
        raise ValidateError
    if not value or ' ' in value:
        raise ValidateError
    return str(value)

def path_check(value):
    '''Very basic requirements for path string'''
    if not isinstance(value, str):
        raise ValidateError
    if not value[0] == '/':
        raise ValidateError
    return str(value)

def get_config():
    '''Create a configobj instance plus validation of user config'''
    spec_file = path.join(path.dirname(module_file), 'config',
                          'configspec.ini')
    config_dir = path.join(path.expanduser('~'), '.derailleur')
    config_path = path.join(config_dir, 'derailleur.conf')
    if not path.isdir(config_dir):
        makedirs(config_dir)
    if not path.isfile(config_path):
        config = ConfigObj(config_path, configspec=spec_file, stringify=False,
                           write_empty_values=True)
    else:
        config = ConfigObj(config_path, configspec=spec_file, stringify=True)
    validator = Validator({'nestring': nestring_check, 'path': path_check})
    validation = config.validate(validator, copy=True)
    if not path.isfile(config_path):
        config.write()
        print('New config file: ' + config_path)
        sys.exit()
    config.dir = config_dir
    config.path = config_path
    config.db = path.join(config_dir, 'derailleur.db')
    config.log = {
        'Add': path.join(config_dir, 'derailleur-add.log'),
        'Feed': path.join(config_dir, 'derailleur-feed.log'),
        'Manager': path.join(config_dir, 'derailleur-manager.log'),
        'Postproc': path.join(config_dir, 'derailleur-postproc.log')
        }
    if validation is True:
        validation = {
            'Connection': True,
            'Add': True,
            'Feed': True,
            'Manager': True,
            'Postproc': True,
            'Audio': True,
            'Smtp': True,
            'Kodi': True
            }
    return (config, validation)
