# -*- coding: utf-8 -*-


# Copyright 2016-2017 Mads Michelsen (mail@brokkr.net)
# This file is part of Derailleur.
# Derailleur is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License,
# or (at your option) any later version.

"""A module implmenting a subclassed version of the Logger class. Our version
   pins some preferences to the instance and auto-adds any needed/requested
   handlers, e.g. null, file and smtp."""

import logging
from logging import handlers


class Logger(logging.Logger):
    '''Custom Logger class with auto-added handlers'''
    def __init__(self, name, config, validation, level=logging.DEBUG, verbose=False):
        super(self.__class__, self).__init__(name, level)
        self.config = config
        self.add_null_handler()
        if self.config[self.name]['file_log']:
            self.add_file_handler()
        if self.config[self.name]['mail_log'] and validation['Smtp'] is True:
            self.add_smtp_handler()
        if verbose:
            self.add_stream_handler()

    def add_null_handler(self):
        '''Adding a basic null_handler'''
        null_handler = logging.NullHandler()
        self.addHandler(null_handler)

    def add_stream_handler(self):
        '''Adding a basic stream_handler'''
        stream_handler = logging.StreamHandler()
        stream_handler.setLevel(logging.DEBUG)
        self.addHandler(stream_handler)

    def add_file_handler(self):
        '''Adding a filehandler'''
        file_handler = logging.FileHandler(self.config.log[self.name])
        file_handler.setLevel(logging.INFO)
        file_formatter = logging.Formatter("%(asctime)s %(message)s",
                                           datefmt='%Y-%m-%d %H:%M')
        file_handler.setFormatter(file_formatter)
        self.addHandler(file_handler)

    def add_smtp_handler(self):
        '''Adding an SMTP_handler'''
        subject = 'derailleur-' + self.name.lower()
        smtp_handler = handlers.SMTPHandler((self.config['Smtp']['mailhost'],
                                             self.config['Smtp']['mailport']),
                                            self.config['Smtp']['fromaddr'],
                                            [self.config['Smtp']['toaddr']],
                                            subject,
                                            credentials=\
                                            (self.config['Smtp']['login'],
                                             self.config['Smtp']['password']),
                                            secure=())
        smtp_handler.setLevel(logging.INFO)
        mail_formatter = logging.Formatter("%(message)s")
        smtp_handler.setFormatter(mail_formatter)
        self.addHandler(smtp_handler)
