# -*- coding: utf-8 -*-

# Copyright 2016-2017 Mads Michelsen (mail@brokkr.net)
# This file is part of Derailleur.
# Derailleur is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License,
# or (at your option) any later version.

"""A library for managing active and completed torrents"""

from datetime import datetime
from collections import namedtuple


Incomplete = namedtuple('non_completed', 'all downloading paused')
SubSeeded = namedtuple('sub_ratio_seeded', 'all seeding paused')
SuperSeeded = namedtuple('ratio_seeded', 'all seeding paused')

class Manager():
    '''Transmission session manager that sorts torrents according to status'''
    def __init__(self, config, client, logger, jar):
        self.client = client
        self.config = config
        self.logger = logger
        self.jar = jar
        torrents = client.get_torrents()
        self.all = torrents
        for torrent in torrents:
            torrent.update()
            torrent.private = 'Private' if torrent._fields['isPrivate'].value \
                              else 'Public'
        incomplete = [x for x in torrents if x.progress < 100]
        incomplete_dl = [x for x in incomplete if x.status == 'downloading']
        incomplete_paused = [x for x in incomplete if x.status == 'stopped']
        self.incomplete = Incomplete(incomplete,
                                     incomplete_dl,
                                     incomplete_paused)
        completed = [x for x in torrents if x.progress == 100]
        sub_seeded = [x for x in completed if x.ratio < x.seed_ratio_limit]
        sub_seeding = [x for x in sub_seeded if x.status == 'seeding']
        sub_paused = [x for x in sub_seeded if x.status == 'stopped']
        self.sub_seeded = SubSeeded(sub_seeded,
                                    sub_seeding,
                                    sub_paused)
        super_seeded = [x for x in completed if x.ratio >= x.seed_ratio_limit]
        super_seeding = [x for x in super_seeded if x.status == 'seeding']
        super_paused = [x for x in super_seeded if x.status == 'stopped']
        self.super_seeded = SuperSeeded(super_seeded,
                                        super_seeding,
                                        super_paused)

    def check_torrents(self):
        '''Loop through various collections and check on them'''
        self.logger.debug("=== DOWNLOADING ===")
        self.logger.debug('{: <32.32s}     type      current'.format(''))
        for torrent in self.incomplete.all:
            self.print_downloading(torrent)
        self.logger.debug('')
        self.logger.debug("=== BELOW RATIO ===")
        self.logger.debug('{: <32.32s}     type      current'.format(''))
        for torrent in self.sub_seeded.all:
            self.print_sub_seeded(torrent)
        self.logger.debug('')
        self.logger.debug("=== ABOVE RATIO: SEEDING ===")
        self.logger.debug('{: <32.32s}     type      current   last'.format(''))
        for torrent in self.super_seeded.seeding:
            self.check_super_seeding(torrent)
        self.logger.debug('')
        self.logger.debug("=== ABOVE RATIO: PAUSED ===")
        self.logger.debug('{: <32.32s}     type      current    hour    ttl'.format(''))
        for torrent in self.super_seeded.paused:
            self.check_super_paused(torrent)
        self.cleanup()

    def print_downloading(self, torrent):
        '''Print info on dl torrents'''
        stream = '{: <32.32s}     {: <7.7s}   {:3.2f}'
        stream = stream.format(torrent.name, torrent.private, torrent.ratio)
        self.logger.debug(stream)

    def print_sub_seeded(self, torrent):
        '''Print info on seed torrents not yet at ratio'''
        stream = '{: <32.32s}     {: <7.7s}   {:3.2f}'
        stream = stream.format(torrent.name, torrent.private, torrent.ratio)
        self.logger.debug(stream)

    def check_super_seeding(self, torrent):
        '''Check up on torrents still seeding after reaching ratio'''
        jar_dic = self.jar.load()
        torrent_hash = torrent.hashString
        if torrent_hash in jar_dic:
            last_seed_stop = jar_dic[torrent_hash][1]
        else:
            last_seed_stop = torrent.seed_ratio_limit
        # /stream logger
        stream = '{: <32.32s}     {: <7.7s}   {:3.2f}      {:3.2f}'
        stream = stream.format(torrent.name, str(torrent.private),
                               torrent.ratio, last_seed_stop)
        self.logger.debug(stream)
        # stream logger/
        stop_tests = {seed_stop: last_seed_stop < float(seed_stop) \
                      <= torrent.ratio for seed_stop in \
                      self.config['Manager']['seed_stops']}
        true_seed_stops = [x for x in stop_tests if stop_tests[x] is True]
        if not true_seed_stops:
            jar_dic[torrent_hash] = (None, last_seed_stop, torrent.name)
            self.jar.save(jar_dic)
            return
        torrent.stop()
        self.logger.info("We have stopped seeding %s @ ratio %0.2f." %
                         (torrent.name, torrent.ratio))
        jar_dic[torrent_hash] = (datetime.now(), torrent.ratio, torrent.name)
        self.jar.save(jar_dic)

    def check_super_paused(self, torrent):
        '''Check up on torrents that have paused after reaching ratio
           (or have been paused because they've reached a seed stop)'''
        jar_dic = self.jar.load()
        torrent_hash = torrent.hashString
        if not torrent_hash in jar_dic:
            self.logger.info("%s has stopped seeding after reaching seed "
                             "limit %0.2f." % (torrent.name, torrent.ratio))
            jar_dic[torrent_hash] = (datetime.now(), torrent.ratio, torrent.name)
            _success = self.jar.save(jar_dic)
            return
        paused_datetime = jar_dic[torrent_hash][0]
        if paused_datetime:
            time_stopped = datetime.now() - jar_dic[torrent_hash][0]
            hours_paused = int(time_stopped.total_seconds() / 3600)
        else:
            hours_paused = 0
            jar_dic[torrent_hash] = (datetime.now(), torrent.ratio, torrent.name)
            _success = self.jar.save(jar_dic)
        wait_hours = int(self.config['Manager']['hours_before_removal'])
        ttl = wait_hours - hours_paused
        # /stream logger
        stream = '{: <32.32s}     {: <7.7s}   {:3.2f}      {:3.0f}     {:3.0f}'
        stream = stream.format(torrent.name, torrent.private, torrent.ratio,
                               hours_paused, ttl)
        self.logger.debug(stream)
        # stream logger/
        if hours_paused >= wait_hours:
            self.client.remove_torrent(torrent.id, delete_data=\
                                       self.config['Manager']['delete_data'])
            self.logger.info("%s has been removed after pausing for %i hours. "
                             "It seeded to %0.2f."
                             % (torrent.name, hours_paused, torrent.ratio))
            del(jar_dic[torrent_hash])
            _success = self.jar.save(jar_dic)

    def cleanup(self):
        '''Cleanup routine that removes leftover records (e.g. for
           entries that have been manually removed)'''
        active_hashes = set([torrent.hashString for torrent in self.all])
        jar_dic = self.jar.load()
        jar_hashes = set(jar_dic.keys())
        difference = jar_hashes.difference(active_hashes)
        for torrent_hash in difference:
            del(jar_dic[torrent_hash])
            _success = self.jar.save(jar_dic)
