# -*- coding: utf-8 -*-

# Copyright 2016-2017 Mads Michelsen (mail@brokkr.net)
# This file is part of Derailleur.
# Derailleur is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License,
# or (at your option) any later version.

"""Defining the stats class"""

import os
import re


class Stats:
    """File stats useful for identification"""

    def create_file_dic(self, folder_path, f):
        """Run once per file, returns all general information about file"""
        file_dic = {}
        file_dic['filename'] = f
        file_dic['folderpath'] = folder_path
        file_dic['parentdir'] = os.path.split(folder_path)[1]
        file_dic['filepath'] = os.path.join(folder_path, f)
        file_dic['basename'], file_dic['ext'] = os.path.splitext(f)
        file_dic['bytesize'] = os.path.getsize(file_dic['filepath'])
        file_type = [k for k, v in list(self.ext_dic.items()) \
                     if file_dic['ext'] in v]
        file_dic['filetype'] = ''.join(file_type)
        return file_dic

    def identify(self, keywords):
        """Analyzes naming and file stats to determine category of files"""
        self.identity = 'generic'
        for keyword in keywords:
            re_keyword = re.search('^' + keyword + ' ', self.torrent_name)
            if re_keyword:
                self.identity = keyword
                return
        if self.torrent_name[0:10] == 'audiobook ':
            self.identity = 'audiobook'
        elif self.type_size_pct['music'] > 80:
            self.identity = 'music'
        elif self.type_size_pct['video'] > 80:
            # tv torrent names either contain the season or season and episode
            re_tv = re.compile('(?i).*season.*|.*series.*|.*complete.*|'
                               '.*s\d{1,2}.*|.*s\d{1,2}e\d{1,2}.*|'
                               '.*(\d{1,2})x(\d\d)')
            re_daily = re.compile('.*(20[0-2][0-9]\.[0-1][0-9]\.[0-3][0-9]).*')
            # films torrent names (almost) always contain a year (1930-2019)
            re_film = re.compile('.*(19[3-9][0-9]|20[0-2][0-9]).*')
            if re_tv.match(self.torrent_name):
                self.identity = 'tv'
            elif re_daily.match(self.torrent_name):
                self.identity = 'tv'
            elif re_film.match(self.torrent_name):
                self.identity = 'film'

    def __init__(self, keywords, torrent_info):
        """Creates instance of Stats class"""
        self.torrent_name = torrent_info.name
        self.torrent_ppath = torrent_info.ppath
        self.torrent_path = os.path.join(torrent_info.ppath, torrent_info.name)
        # what types are various file extensions
        self.ext_dic = {
            'package': ['.apk', '.deb', '.rpm'],
            'music': ['.mp3', '.ogg', '.flac', '.wma', '.m4a'],
            'video': ['.avi', '.mkv', '.wmv', '.mp4', '.3gp'],
            'text': ['.txt', '.nfo', '.sfv', '.idx', '.url', '.m3u'],
            'book': ['.pdf', '.epub', '.mobi'],
            'subtitle': ['.srt', '.smi', '.sub'],
            'image': ['.jpg', '.jpeg', '.png', '.gif', '.bmp'],
            'compressed': ['.zip', '.rar', '.gz'],
            'disc': ['.iso'],
            'part': ['.part']
            }
        for i in range(0, 100):
            self.ext_dic['compressed'].append(".r%02d" % i)
        # initialize containers
        self.file_lst, self.file_by_type_dic = [], {}
        self.type_count, self.type_size = {}, {}
        self.type_count_pct, self.type_size_pct = {}, {}
        # tree walking
        if os.path.isfile(self.torrent_path):
            file_dic = self.create_file_dic(torrent_info.ppath, torrent_info.name)
            self.file_lst.append(file_dic)
        else:
            for (root, dirs, files) in os.walk(self.torrent_path):
                dir_contents = [self.create_file_dic(root, f) for f in files]
                self.file_lst.extend(dir_contents)
        # discount .part files
        self.file_lst = [file_dic for file_dic in self.file_lst if
                         file_dic['filetype'] != 'part']
        # calculate overall torrent stats
        self.torrent_count = len(self.file_lst)
        self.torrent_size = sum([fdic['bytesize'] for fdic in self.file_lst])
        if not self.torrent_count:
            self.torrent_count = 1
        if not self.torrent_size:
            self.torrent_size = 1
        # calculate stats for represented file_types
        for file_type in self.ext_dic:
            file_type_lst = [file_dic for file_dic in self.file_lst \
                if file_dic['filetype'] == file_type]
            self.file_by_type_dic[file_type] = file_type_lst
            self.type_count[file_type] = len(file_type_lst)
            self.type_size[file_type] = sum([file_dic['bytesize'] \
                for file_dic in file_type_lst])
            self.type_count_pct[file_type] = float(self.type_count[file_type]) \
                / float(self.torrent_count) * 100
            self.type_size_pct[file_type] = float(self.type_size[file_type]) \
                / float(self.torrent_size) * 100

        self.identify(keywords)
