# -*- coding: utf-8 -*-

# Copyright 2016-2017 Mads Michelsen (mail@brokkr.net)
# This file is part of Derailleur.
# Derailleur is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License,
# or (at your option) any later version.

"""Defining the TV class"""

import os
import re
from . import generic


class Torrent(generic.Torrent):
    """TV specific content type. Establishes series name/season number
    for kodi friendly folder scheme (series name/season no/files).
    Works for series, seasons and individual episodes alike."""

    def manipulate(self):
        """Use regexes to find series name and season number, create folder
        scheme, filter junk files (anything but 'real' video and subs)"""
        # lowercase, replace points by spaces, remove multiple occurences
        tname = self.torrent_name.lower().replace(".", " ").replace("_", " ")
        tname = re.sub(' +', ' ', tname)
        # find indices for common title termination indicators
        re_sxex = re.search(r's(\d\d)e(\d\d)|(\d{1,2})x(\d\d)', tname)
        re_season = re.search(r'season (\d+)|series (\d+)|s(\d\d)', tname)
        re_complete = re.search(r'complete', tname)
        re_year = re.search(r'(19[3-9]\d|20[0-1]\d)', tname)
        searches = [re_sxex, re_season, re_complete, re_year]
        re_indices = [result.start() for result in searches if result]
        # use lowest index to get title
        if not re_indices:
            re_indices = [-1]
        lowest_index = min(re_indices)
        series_name = tname[0:lowest_index]
        # clean up series name (allow closing parenthesis for '(US)' etc.)
        self.series_name = re.match(r'.*[^\-\_\ \(]', series_name).group()
        # get fallback data from torrent name and set series/episode destination
        if re_sxex:
            se = [int(result) for result in re_sxex.groups() if result]
            season_no, episode_no = se[0], se[1]
            type_path = os.path.join(self.type_path, 'episodes')
        elif re_season:
            season_no = [int(result) for result in re_season.groups() if result][0]
            type_path = os.path.join(self.type_path, 'series')
        else:
            season_no, episode_no = (None, None)
            type_path = os.path.join(self.type_path, 'series')
        # get video files and subtitles, ignore the rest
        keep_types = ['video', 'subtitles']
        keepers = [f for f in self.stats.file_lst if f['filetype'] in keep_types]
        for file_dic in keepers:
            file_name = file_dic['filename'].lower()
            if re.search(r'sample', file_name):
                continue
            old_file_path = file_dic['filepath']
            new_folder_path = os.path.join(type_path, self.series_name)
            # override season_no on a per-episode basis
            re_f_sxex = re.search(r's(\d\d)e(\d\d)', file_name)
            re_f_exe = re.search(r'(\d\d)x(\d\d)', file_name)
            if re_f_exe:
                season_no = int(re_f_exe.group(1))
            if re_f_sxex:
                season_no = int(re_f_sxex.group(1))
            if season_no:
                season = 'season ' + str(season_no)
                new_folder_path = os.path.join(new_folder_path, season)
            new_file_path = os.path.join(new_folder_path, file_dic['filename'])
            self.move_lst.append((old_file_path, new_file_path))
            self.move_type_lst.append(file_dic['filetype'])
